
class ShopifyAppMiddleware:

    def __init__(self, get_response) -> None:
        self.get_response = get_response

    def __call__(self, request):

        response = self.get_response(request)

        return response



from django.apps import apps
from django.http import HttpResponse
from django.urls import reverse
from django.shortcuts import redirect
from django.conf import settings

from shopify import ApiAccess, Session, session_token
from shopify.utils import shop_url

from .utils import get_auth_shop, get_shop, get_shop_model


def shopify_embed(func):
    def wrapper(request, **kwargs):
        shop = request.GET.get("shop")
        response = func(request, **kwargs)
        if shop:
            ancestors = f"frame-ancestors https://{shop} " "https://admin.shopify.com"
            response["Content-Security-Policy"] = ancestors

        return response

    return wrapper


def shop_session(func):
    def wrapper(*args, **kwargs):
        try:
            request = args[0]

            authorization_header = request.META.get("HTTP_SHOPIFYAUTH")
            shop_shopify_domain = request.META.get("HTTP_SHOPSHOPIFYDOMAIN")
            auth_shop = get_auth_shop(shop_shopify_domain)
            print(auth_shop)

            if (
                not authorization_header
                and request.user.is_authenticated
                and request.user.is_staff
            ):
                print("user is staff ")

            decoded_session_token = session_token.decode_from_header(
                authorization_header=authorization_header,
                api_key=auth_shop.shopify_app_api_key,
                secret=auth_shop.shopify_app_api_secret,
            )

            shopify_domain = decoded_session_token.get("dest")
            shopify_domain = shopify_domain.removeprefix("https://")

            check_shop_domain(request, kwargs, shopify_domain)
            check_shop_known(request, kwargs)

            kwargs["shopify_user_id"] = decoded_session_token["sub"]

            return func(*args, **kwargs)

        except session_token.SessionTokenError as e:
            print(e)
            return HttpResponse(status=401, content=str(e))

    return wrapper


def shopify_session(session_token):
    shopify_domain = session_token.get("dest").removeprefix("https://")
    api_version = apps.get_app_config("shopify_app").SHOPIFY_API_VERSION
    access_token = (
        get_shop_model().objects.get(shopify_domain=shopify_domain).shopify_token
    )

    return Session.temp(shopify_domain, api_version, access_token)


def known_shop_required(func):
    def wrapper(*args, **kwargs):
        request = args[0]
        try:
            shop_domain = request.GET.get("shop", request.POST.get("shop"))
            check_shop_domain(request, kwargs, shop_domain)
            check_shop_known(request, kwargs)
        except Exception as e:
            print(e)
            raise ValueError("Shop must be known")
        finally:
            return func(*args, **kwargs)

    return wrapper


def check_shop_domain(request, kwargs, shop_domain):
    shop_domain = get_sanitized_shop_param(shop_domain)
    kwargs["shopify_domain"] = shop_domain
    request.shopify_domain = shop_domain
    return shop_domain


def get_sanitized_shop_param(shop_domain):
    sanitized_shop_domain = shop_url.sanitize_shop_domain(shop_domain)
    if not sanitized_shop_domain:
        raise ValueError("Shop must match 'example.myshopify.com'")
    return sanitized_shop_domain


def check_shop_known(request, kwargs):
    shop = get_shop_model().objects.get(shopify_domain=kwargs.get("shopify_domain"))
    kwargs["shop"] = shop
    request.shop = shop


def latest_access_scopes_required(func):
    def wrapper(*args, **kwargs):
        shop = kwargs.get("shop")

        try:
            configured_access_scopes = apps.get_app_config(
                "shopify_app"
            ).SHOPIFY_API_SCOPES
            current_access_scopes = shop.access_scopes

            assert ApiAccess(configured_access_scopes) == ApiAccess(
                current_access_scopes
            )
        except:
            kwargs["scope_changes_required"] = True

        return func(*args, **kwargs)

    return wrapper


def app_proxy_view(func):
    def wrapper(*args, **kwargs):
        request = args[0]

        shop_domain = request.META.get("HTTP_X_SHOP_DOMAIN")
        assert shop_domain is not None, "Shop domain is required"

        shop = get_shop(shop_domain)
        assert shop.proxy_request_is_valid(request), "Proxy request is invalid"

        kwargs["shop"] = shop

        return func(*args, **kwargs)

    return wrapper



import json
import requests

from django.views import View
from django.urls import reverse
from django.conf import settings
from django.shortcuts import redirect, render, HttpResponse
from django.views.decorators.clickjacking import xframe_options_exempt
from django.utils.decorators import method_decorator

from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.exceptions import APIException

import shopify

from .utils import (
    get_function_from_string,
    get_shop_model,
    webhook_request_is_valid,
    get_shop,
    get_auth_shop,
)
from .decorators import shopify_embed


class ShopAPIView(APIView):
    pass


def create_permission_url(shop_url, redirect_path="", redirect_path_name=""):
    auth_shop = get_auth_shop(shop_url)
    shopify.Session.setup(
        api_key=auth_shop.shopify_app_api_key,
        secret=auth_shop.shopify_app_api_secret,
    )

    api_version = "2022-07"
    state = ""

    if redirect_path_name:
        path = reverse(redirect_path_name)
    elif redirect_path:
        path = redirect_path

    redirect_uri = f"{settings.SHOPIFY_APP_HOST}{path}"
    scopes = settings.SHOPIFY_APP_SCOPES

    newSession = shopify.Session(shop_url, api_version)
    auth_url = newSession.create_permission_url(scopes, redirect_uri, state)
    return auth_url


class InitTokenRequestView(View):
    redirect_path_name = ""
    shop_not_provided_redirect_path_name = ""

    @method_decorator(xframe_options_exempt)
    @method_decorator(shopify_embed)
    def get(self, request, *args, **kwargs):
        shop_url = request.GET.get("shop")

        if not shop_url:
            if self.shop_not_provided_redirect_path_name:
                return redirect(self.shop_not_provided_redirect_path_name)
            else:
                message = "No shop provided, please open this app from Shopify"
                return HttpResponse(
                    message,
                    status=404,
                )

        auth_url = create_permission_url(
            shop_url, redirect_path_name=self.redirect_path_name
        )
        auth_shop = get_auth_shop(shop_url)

        if request.GET.get("embedded"):
            context = {
                "api_key": auth_shop.shopify_app_api_key,
                "host": request.GET.get("host"),
                "redirect": auth_url,
            }
            return render(request, "shopify_app/index.html", context=context)
        else:
            return redirect(auth_url)


def request_personal_access_token(shop, client_id, client_secret, authorization_code):
    response = requests.post(
        (
            f"https://{shop}/admin/oauth/access_token"
            f"?client_id={client_id}&client_secret={client_secret}&"
            f"code={authorization_code}"
        )
    )
    if response.status_code != 200:
        raise Exception("Could not request access token")

    return response.json()


class EndTokenRequestView(View):
    redirect_path_name = ""

    def get(self, request, *args, **kwargs):
        shop_url = request.GET.get("shop")
        auth_shop = get_auth_shop(shop_url)

        shopify.Session.setup(
            api_key=auth_shop.shopify_app_api_key,
            secret=auth_shop.shopify_app_api_secret,
        )

        if not shopify.Session.validate_params(request.GET.dict()):
            raise Exception("Invalid HMAC: Possibly malicious login")

        data = request_personal_access_token(
            shop_url,
            auth_shop.shopify_app_api_key,
            auth_shop.shopify_app_api_secret,
            request.GET.get("code"),
        )

        Shop = get_shop_model()
        shop_record = Shop.objects.get_or_create(shopify_domain=shop_url)[0]

        if data.get("associated_user"):
            shop_record.on_user_login(data["associated_user"], request=request)

            host = settings.SHOPIFY_APP_HOST
            path = reverse(self.redirect_path_name)
            query = request.META["QUERY_STRING"]

            return redirect(f"{host}{path}?{query}")
        else:
            access_token = data.pop("access_token")

        installed = bool(not shop_record.shopify_token and access_token)

        shop_record.shopify_token = access_token

        access_scopes = data["scope"]
        access_scopes = access_scopes[0:249]
        shop_record.access_scopes = access_scopes

        shop_record.save()

        if installed:
            shop_record.installed(request=request)

        auth_url = create_permission_url(shop_url, redirect_path=request.path)
        return redirect(f"{auth_url}&grant_options[]=per-user")


class WebhookAuthenticationFailed(APIException):
    status_code = 401
    default_detail = "authentication failed"


class MandatoryWebhooksView(APIView):
    topic = None

    def post(self, request, *args, **kwargs):
        domain = request.META["HTTP_X_SHOPIFY_SHOP_DOMAIN"]
        hmac_header = self.request.META.get("HTTP_X_SHOPIFY_HMAC_SHA256")

        try:
            shop = get_shop(domain)
        except:
            raise WebhookAuthenticationFailed()

        if not webhook_request_is_valid(shop, hmac_header, request.body):
            raise WebhookAuthenticationFailed()

        func = get_function_from_string(settings.SHOPIFY_GDPR_WEBHOOK_CALLBACK)
        func(shop, self.topic, request.data, attributes={})

        return Response(status=200)


class WebhooksView(APIView):
    def post(self, request, *args, **kwargs):
        meta = request.META
        shop = get_shop(meta["HTTP_X_SHOPIFY_SHOP_DOMAIN"])
        hmac = meta["HTTP_X_SHOPIFY_HMAC_SHA256"]
        webhook_topic = meta["HTTP_X_SHOPIFY_TOPIC"]
        api_version = meta["HTTP_X_SHOPIFY_API_VERSION"]
        data = request.body

        if webhook_request_is_valid(shop, hmac, data):
            callback_path = settings.SHOPIFY_WEBHOOK_CALLBACK
            data = json.loads(data.decode("utf-8"))
            func = get_function_from_string(callback_path)
            attributes = {
                "api_version": api_version,
            }
            func(shop, webhook_topic, data, attributes=attributes)
            return Response(status=200)
        else:
            raise WebhookAuthenticationFailed()



import base64
import hashlib
import hmac
import importlib
from django.apps import apps
from django.conf import settings


def get_shop_model():
    Model = apps.get_model(settings.SHOPIFY_SHOP_MODEL)
    return Model


def get_shop(shopify_domain):
    Model = get_shop_model()
    shop = Model.objects.get(shopify_domain=shopify_domain)
    return shop


def get_auth_shop(shopify_domain):
    Model = get_shop_model()
    try:
        shop = Model.objects.get(shopify_domain=shopify_domain)
    except Model.DoesNotExist:
        shop = Model()

    return shop


def get_function_from_string(string):
    func_name = string.rsplit(".")[-1]
    location = string.replace(f".{func_name}", "")
    module = importlib.import_module(location)
    if not hasattr(module, func_name):
        raise AttributeError(f"Module {module} does not have function {func_name}")
    func = getattr(module, func_name)
    return func


def webhook_request_is_valid(shop, received_hmac, message):
    secret = shop.shopify_app_api_secret.encode("utf-8")
    digest = hmac.new(secret, message, hashlib.sha256).digest()
    computed_hmac = base64.b64encode(digest)
    received_hmac = received_hmac.encode("utf-8")
    is_valid = hmac.compare_digest(computed_hmac, received_hmac)
    return is_valid


def crm_update_all_shops_billing_status(
    update_shop_billing_status_task=None,
    delay_between_tasks=10,
):
    Model = get_shop_model()
    shops = Model.objects.all()
    index = 0
    for shop in shops:
        if update_shop_billing_status_task:
            update_shop_billing_status_task.apply_async(
                args=[shop.id], countdown=delay_between_tasks * index
            )
        else:
            crm_update_shop_billing_status(shop.id)

        index += 1


def crm_update_shop_billing_status(shop_id):
    Model = get_shop_model()
    shop = Model.objects.get(id=shop_id)

    charge_price = 0
    response = shop.get(
        f"/admin/api/{shop.api_version}/recurring_application_charges.json?status=active"
    )

    if response.status_code == 200:
        charges = response.json()["recurring_application_charges"]
        for charge in charges:
            if charge["status"] == "active" and charge["test"] == False:
                charge_price = float(charge["price"])
                break

    shop.crm_on_billing_plan_change(charge_price)



import hashlib
import hmac
import json
import base64
from functools import cached_property
from sys import api_version
from django.conf import settings

from django.db import models
from django.apps import apps
import requests

import shopify
from shopify import Session
from crm_events import on_billing_plan_change, on_uninstall, on_login, on_install


from .services.webhooks import update_shop_webhooks
from .exceptions import ShopifyRequestException


class ShopBase(models.Model):
    shopify_domain = models.CharField(max_length=50, default="")
    shopify_token = models.CharField(max_length=150, default="", blank=True, null=True)
    access_scopes = models.CharField(max_length=250, default="")

    shopify_app_api_key_overwrite = models.CharField(max_length=50, default="")
    shopify_app_api_secret_overwrite = models.CharField(max_length=50, default="")

    def get(self, path):
        url = f"https://{self.shopify_domain}{path}"
        return requests.get(url, headers=self.request_headers)

    def post(self, path, data):
        url = f"https://{self.shopify_domain}{path}"
        return requests.post(url, headers=self.request_headers, data=json.dumps(data))

    def put(self, path, data):
        url = f"https://{self.shopify_domain}{path}"
        return requests.put(url, headers=self.request_headers, data=json.dumps(data))

    @property
    def request_headers(self):
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "X-Shopify-Access-Token": self.shopify_token,
        }
        return headers

    def graphql(self, query=None, variables=None):
        url = f"https://{self.shopify_domain}/admin/api/{self.api_version}/graphql.json"
        data = {"query": query}

        if variables:
            data["variables"] = variables

        return requests.post(url, headers=self.request_headers, json=data)

    @cached_property
    def shopify(self):
        return shopify

    @property
    def shopify_app_api_key(self):
        if self.shopify_app_api_key_overwrite:
            return self.shopify_app_api_key_overwrite
        else:
            return apps.get_app_config("shopify_app").SHOPIFY_API_KEY

    @property
    def shopify_app_api_secret(self):
        if self.shopify_app_api_secret_overwrite:
            return self.shopify_app_api_secret_overwrite
        else:
            return apps.get_app_config("shopify_app").SHOPIFY_API_SECRET

    @property
    def shopify_session(self):
        api_version = apps.get_app_config("shopify_app").SHOPIFY_API_VERSION
        shopify_domain = self.shopify_domain
        return Session.temp(shopify_domain, api_version, self.shopify_token)

    @cached_property
    def api_version(self):
        return apps.get_app_config("shopify_app").SHOPIFY_API_VERSION

    def installed(self, request=None):
        pass

    def update_webhooks(self):
        update_shop_webhooks(self)

    def graph(self, operation_name, variables, operations_document):
        with self.shopify_session:
            result = shopify.GraphQL().execute(
                query=operations_document,
                variables=variables,
                operation_name=operation_name,
            )

        result = json.loads(result)
        return result

    @cached_property
    def host(self):
        admin_url = f"{self.shopify_domain}/admin"
        return base64.b64encode(admin_url.encode()).decode()

    def on_user_login(self, user_data, request=None):
        pass

    def proxy_request_is_valid(self, request):
        return self.validate_request_hmac(
            request, separator="", signature_name="signature"
        )

    def request_shopify_hmac_is_valid(self, request):
        return self.validate_request_hmac(request)

    def validate_request_hmac(self, request, separator="&", signature_name="hmac"):

        params = request.GET.dict()
        myhmac = params.pop(signature_name)
        line = f"{separator}".join(
            ["%s=%s" % (key, value) for key, value in sorted(params.items())]
        )
        api_secret = self.shopify_app_api_secret
        api_secret = api_secret.encode("utf-8")
        h = hmac.new(api_secret, line.encode("utf-8"), hashlib.sha256)

        return hmac.compare_digest(h.hexdigest(), myhmac)

    def get_shop_data(self):
        response = self.get(f"/admin/api/{self.api_version}/shop.json")
        if response.status_code == 200:
            shop_data = response.json()["shop"]
            return shop_data

        raise ShopifyRequestException(response.status_code, response.text)

    def get_crm_shop_data(self):
        shop_name, domain = self.shopify_domain, self.shopify_domain
        email, phone, country = "", "", ""

        try:
            shop_data = self.get_shop_data()
            shop_name = shop_data["name"]
            email = shop_data["email"]
            phone = shop_data["phone"]
            country = shop_data["country_code"]
        except ShopifyRequestException:
            pass

        return {
            "name": shop_name,
            "domain": domain,
            "email": email,
            "phone": phone,
            "country": country,
        }

    def crm_on_billing_plan_change(self, plan_price):
        on_billing_plan_change(self.get_crm_shop_data(), plan_price)

    def crm_on_uninstall(self, users=[]):
        on_uninstall(self.get_crm_shop_data(), users)

    def crm_on_login(self, user_data):
        on_login(self.get_crm_shop_data(), user_data)

    def crm_on_install(self):
        on_install(self.get_crm_shop_data())

    class Meta:
        abstract = True



from django.core.management import BaseCommand, CommandError

from ...utils import get_shop_model


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("shopify_subdomain", type=str)

    def execute(self, *args, **options):
        Shop = get_shop_model()
        shopify_domain = f"{options['shopify_subdomain']}.myshopify.com"

        try:
            shop = Shop.objects.get(shopify_domain=shopify_domain)
        except Shop.DoesNotExist:
            raise CommandError("Shop does not exist")

        shop.update_webhooks()



from django.urls import path, include

from . import views

app_name = 'shopify_app'

urlpatterns = [
    path(
        'gdpr-webhooks/customer-data-request',
        views.MandatoryWebhooksView.as_view(topic='customer_data_request')
    ),
    path(
        'gdpr-webhooks/customer-data-erasure',
        views.MandatoryWebhooksView.as_view(topic='customer_data_erasure')
    ),
    path(
        'gdpr-webhooks/shop-data-erasure',
        views.MandatoryWebhooksView.as_view(topic='shop_data_erasure')
    ),
    path('webhooks', views.WebhooksView.as_view(), name='webhooks'),
]



from django.conf import settings

from django.apps import apps
from django.urls import reverse


def update_shop_webhooks(shop):
    config = apps.get_app_config("shopify_app")

    with shop.shopify_session:
        path = reverse("shopify_app:webhooks")
        address = f"{config.WEBHOOK_HOST}{path}"

        for topic in settings.SHOPIFY_WEBHOOK_TOPICS:
            webhooks = shop.shopify.Webhook.find(topic=topic)

            if len(webhooks):
                webhook = webhooks[0]
                webhook.address = address
                webhook.save()

            else:
                webhook = shop.shopify.Webhook.create(
                    {"topic": topic, "address": address, "format": "json"}
                )

        print(shop.shopify.Webhook.find())

        for created_webhook in shop.shopify.Webhook.find():
            if created_webhook.topic not in settings.SHOPIFY_WEBHOOK_TOPICS:
                created_webhook.destroy()



from django.apps import AppConfig
from django.conf import settings


class ShopifyAppConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "shopify_app"

    SHOPIFY_API_KEY = settings.SHOPIFY_API_KEY
    SHOPIFY_API_SECRET = settings.SHOPIFY_API_SECRET
    APP_URL = settings.SHOPIFY_APP_HOST.replace("https://", "")

    APP_HOST = settings.SHOPIFY_APP_HOST

    WEBHOOK_HOST = getattr(settings, "SHOPIFY_WEBHOOK_HOST", APP_HOST)
    WEBHOOK_TOPICS = getattr(settings, "SHOPIFY_WEBHOOK_TOPICS", [])

    SHOPIFY_API_VERSION = getattr(settings, "SHOPIFY_API_VERSION", "2022-04")
    SHOPIFY_API_SCOPES = getattr(settings, "SHOPIFY_APP_SCOPES", [])

    def ready(self):
        return super().ready()



from setuptools import find_packages, setup

setup(
    name="django_shopify_app",
    version="2.0.5",
    author="Santiago Fernandez",
    author_email="",
    packages=find_packages(),
    scripts=[],
    url="http://pypi.python.org/pypi/django_shopify_app/",
    license="MIT",
    description="A django app with all the tools required to make a Shopify app",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    package_data={"shopify_app": ["templates/shopify_app/*.html"]},
    install_requires=[
        "Django >= 4.0.0",
        "pytest",
        "shopifyapi",
        "django-crm-events",
    ],
)
